@@ -341,16 +341,7 @@ fill_in_new_term_submenus (TerminalWindow *window)
       return;      
     }
 
-  /* FIXME profile name can have an underscore which breaks
-   * the mnemonic stuff
-   */
-  /* avoid "Default (Default)" but have "Default (foobar)" */
-  if (strcmp (terminal_profile_get_visible_name (default_profile),
-              "Default") == 0)
-    str = g_strdup (_("_Default"));
-  else
-    str = g_strdup_printf (_("_Default (%s)"),
-                           terminal_profile_get_visible_name (default_profile));
+  str = g_strdup (terminal_profile_get_visible_name (default_profile));
   
   gtk_widget_set_sensitive (window->priv->new_window_menuitem, TRUE);
   gtk_widget_set_sensitive (window->priv->new_tab_menuitem, TRUE);
@@ -364,7 +355,7 @@ fill_in_new_term_submenus (TerminalWindow *window)
                              new_window_menu);
 
   /* Add default menu item */
-  menu_item = gtk_menu_item_new_with_mnemonic (str);
+  menu_item = gtk_menu_item_new_with_label (str);
   gtk_widget_show (menu_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (new_window_menu),
                          menu_item);
@@ -393,7 +384,7 @@ fill_in_new_term_submenus (TerminalWindow *window)
                              new_tab_menu);
 
   /* Add default menu item */
-  menu_item = gtk_menu_item_new_with_mnemonic (str);
+  menu_item = gtk_menu_item_new_with_label (str);
   gtk_widget_show (menu_item);
   gtk_menu_shell_append (GTK_MENU_SHELL (new_tab_menu),
                          menu_item);
@@ -479,20 +470,6 @@ fill_in_new_term_submenus (TerminalWindow *window)
   g_list_free (profiles);  
 }
 
-static void
-terminal_menu_activated (GtkMenuItem    *menu_item,
-                         TerminalWindow *window)
-{
-  fill_in_config_picker_submenu (window);
-}
-
-static void
-file_menu_activated (GtkMenuItem    *menu_item,
-                     TerminalWindow *window)
-{
-  fill_in_new_term_submenus (window);
-}
-
 static void
 terminal_window_init (TerminalWindow *window)
 {
@@ -541,12 +518,6 @@ terminal_window_init (TerminalWindow *window)
                         "", NULL,
                         NULL, NULL);
   window->priv->file_menuitem = mi;
-
-  /* Set up a callback to demand-create the New Window/New Tab submenus */
-  /* FIXME they need to be created in advance so accels work */
-  g_signal_connect (G_OBJECT (mi), "activate",
-                    G_CALLBACK (file_menu_activated),
-                    window);
   
   menu = gtk_menu_new ();
   gtk_menu_set_accel_group (GTK_MENU (menu),
@@ -630,12 +601,7 @@ terminal_window_init (TerminalWindow *window)
   mi = append_menuitem (window->priv->menubar,
                         "", NULL,
                         NULL, NULL);
-  window->priv->terminal_menuitem = mi;
-  
-  /* Set up a callback to demand-create the Profiles submenu */
-  g_signal_connect (G_OBJECT (mi), "activate",
-                    G_CALLBACK (terminal_menu_activated),
-                    window);  
+  window->priv->terminal_menuitem = mi;  
   
   menu = gtk_menu_new ();
   gtk_menu_set_accel_group (GTK_MENU (menu),
@@ -696,6 +662,8 @@ terminal_window_init (TerminalWindow *window)
                               G_CALLBACK (about_callback), window);
   set_menuitem_text (mi, _("_About GNOME Terminal"),
                      FALSE);
+
+  terminal_window_reread_profile_list (window);
   
   terminal_window_set_menubar_visible (window, TRUE);
   window->priv->use_default_menubar_visibility = TRUE;  
@@ -838,7 +806,8 @@ static void
 profile_set_callback (TerminalScreen *screen,
                       TerminalWindow *window)
 {
-  /* nothing anymore */
+  /* Redo the pick-a-profile menu */
+  fill_in_config_picker_submenu (window);
 }
 
 static void
@@ -1018,11 +987,24 @@ void
 terminal_window_remove_screen (TerminalWindow *window,
                                TerminalScreen *screen)
 {
+  int old_grid_width;
+  int old_grid_height;
+  
   g_return_if_fail (terminal_screen_get_window (screen) == window);
+
+  old_grid_width = -1;
+  old_grid_height = -1;
   
   if (window->priv->active_term == screen)
-    window->priv->active_term = NULL;
+    {
+      GtkWidget *old_active_widget;
+      
+      window->priv->active_term = NULL;
 
+      old_active_widget = terminal_screen_get_widget (screen);
+      terminal_widget_get_size (old_active_widget, &old_grid_width, &old_grid_height);
+    }
+      
   window->priv->terms = g_list_remove (window->priv->terms, screen);
 
   g_signal_handlers_disconnect_by_func (G_OBJECT (screen),
@@ -1052,9 +1034,23 @@ terminal_window_remove_screen (TerminalWindow *window,
   update_notebook (window);
 
   /* Put ourselves back in a sane state */
+  /* FIXME this block never gets called because removing a notebook page
+   * already emitted switch_page and updated the active terminal.
+   */
   if (window->priv->active_term == NULL &&
       window->priv->terms)
-    terminal_window_set_active (window, window->priv->terms->data);
+    {
+      terminal_window_set_active (window, window->priv->terms->data);
+    }
+
+  if (window->priv->active_term)
+    {
+#ifdef DEBUG_GEOMETRY
+      g_print ("setting size with forced grid after removing a terminal\n");
+#endif
+      terminal_window_set_size_force_grid (window, window->priv->active_term,
+                                           TRUE, old_grid_width, old_grid_height);
+    }
 
   reset_tab_menuitems (window);
   update_tab_sensitivity (window);
@@ -1099,6 +1095,9 @@ terminal_window_set_menubar_visible (TerminalWindow *window,
 
   if (window->priv->active_term)
     {
+#ifdef DEBUG_GEOMETRY
+      g_print ("setting size after toggling menubar visibility\n");
+#endif
       terminal_window_set_size (window, window->priv->active_term, TRUE);
     }
 }
@@ -1113,6 +1112,16 @@ void
 terminal_window_set_size (TerminalWindow *window,
                           TerminalScreen *screen,
                           gboolean        even_if_mapped)
+{
+  terminal_window_set_size_force_grid (window, screen, even_if_mapped, -1, -1);
+}
+
+void
+terminal_window_set_size_force_grid (TerminalWindow *window,
+                                     TerminalScreen *screen,
+                                     gboolean        even_if_mapped,
+                                     int             force_grid_width,
+                                     int             force_grid_height)
 {
   /* Owen's hack from gnome-terminal */
   GtkWidget *widget;
@@ -1157,17 +1166,23 @@ terminal_window_set_size (TerminalWindow *window,
 
   terminal_widget_get_cell_size (widget, &char_width, &char_height);
   terminal_widget_get_size (widget, &grid_width, &grid_height);
+
+  if (force_grid_width >= 0)
+    grid_width = force_grid_width;
+  if (force_grid_height >= 0)
+    grid_height = force_grid_height;
+  
   terminal_widget_get_padding (widget, &xpad, &ypad);
   
   w += xpad + char_width * grid_width;
   h += ypad + char_height * grid_height;
 
 #ifdef DEBUG_GEOMETRY
-  g_print ("set size: grid %dx%d setting %dx%d pixels\n",
-           grid_width, grid_height, w, h);
+  g_print ("set size: grid %dx%d force %dx%d setting %dx%d pixels\n",
+           grid_width, grid_height, force_grid_width, force_grid_height, w, h);
 #endif
   
-  if (even_if_mapped)
+  if (even_if_mapped && GTK_WIDGET_MAPPED (app))
     gtk_window_resize (GTK_WINDOW (app), w, h);
   else
     gtk_window_set_default_size (GTK_WINDOW (app), w, h);
@@ -1214,9 +1229,14 @@ terminal_window_set_active (TerminalWindow *window,
                                                         screen_get_hbox (screen)));
 
   /* set size of window to current grid size */
+#ifdef DEBUG_GEOMETRY
+  g_print ("setting size after flipping notebook pages\n");
+#endif
   terminal_window_set_size (window, screen, TRUE);
   
   gtk_widget_grab_focus (terminal_screen_get_widget (window->priv->active_term));
+
+  fill_in_config_picker_submenu (window);
 }
 
 TerminalScreen*
@@ -1313,6 +1333,18 @@ notebook_page_switched_callback (GtkWidget       *notebook,
   GtkWidget* page_widget;
   TerminalScreen *screen;
   GtkWidget *menu_item;
+  int old_grid_width, old_grid_height;
+  GtkWidget *old_widget;
+  
+  old_widget = NULL;
+  old_grid_width = -1;
+  old_grid_height = -1;
+
+  if (window->priv->active_term)
+    {
+      old_widget = terminal_screen_get_widget (window->priv->active_term);
+      terminal_widget_get_size (old_widget, &old_grid_width, &old_grid_height);
+    }
   
   page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
                                            page_num);
@@ -1325,6 +1357,14 @@ notebook_page_switched_callback (GtkWidget       *notebook,
 
   terminal_window_set_active (window, screen);
 
+  /* This is so we maintain the same grid moving among tabs with
+   * different fonts.
+   */
+#ifdef DEBUG_GEOMETRY
+  g_print ("setting size in switch_page handler\n");
+#endif
+  terminal_window_set_size_force_grid (window, screen, TRUE, old_grid_width, old_grid_height);
+  
   update_tab_sensitivity (window);
   
   menu_item = screen_get_menuitem (screen);
@@ -1750,7 +1790,7 @@ new_window_callback (GtkWidget      *menuitem,
   if (!terminal_profile_get_forgotten (profile))
     {
       terminal_app_new_terminal (terminal_app_get (),
-                                 terminal_screen_get_profile (window->priv->active_term),
+                                 profile,
                                  NULL,
                                  FALSE, FALSE, NULL, NULL, NULL, NULL);
     }
@@ -1770,7 +1810,7 @@ new_tab_callback (GtkWidget      *menuitem,
   if (!terminal_profile_get_forgotten (profile))
     {
       terminal_app_new_terminal (terminal_app_get (),
-                                 terminal_screen_get_profile (window->priv->active_term),
+                                 profile,
                                  window,
                                  FALSE, FALSE, NULL, NULL, NULL, NULL);
     }
@@ -2057,3 +2097,61 @@ about_callback (GtkWidget      *menuitem,
   gtk_widget_show (about);
 }
 
+
+static void
+default_profile_changed (TerminalProfile    *profile,
+                         TerminalSettingMask mask,
+                         void               *data)
+{  
+  if (mask & TERMINAL_SETTING_IS_DEFAULT)
+    {
+      TerminalWindow *window;
+
+      window = TERMINAL_WINDOW (data);
+
+      /* When the default changes, we get a settings change
+       * on the old default and the new. We only rebuild
+       * the menu on the notify for the new default.
+       */
+      if (terminal_profile_get_is_default (profile))
+        fill_in_new_term_submenus (window);
+    }
+}
+
+static void
+monitor_profiles_for_is_default_change (TerminalWindow *window)
+{
+  GList *profiles;
+  GList *tmp;
+  
+  profiles = terminal_profile_get_list ();
+
+  tmp = profiles;
+  while (tmp != NULL)
+    {
+      TerminalProfile *profile = tmp->data;
+
+      g_signal_handlers_disconnect_by_func (G_OBJECT (profile),
+                                            G_CALLBACK (default_profile_changed),
+                                            window);
+      
+      g_signal_connect_object (G_OBJECT (profile),
+                               "changed",
+                               G_CALLBACK (default_profile_changed),
+                               G_OBJECT (window),
+                               0);
+      
+      tmp = tmp->next;
+    }
+
+  g_list_free (profiles);
+}
+
+void
+terminal_window_reread_profile_list (TerminalWindow *window)
+{
+  monitor_profiles_for_is_default_change (window);
+  
+  fill_in_config_picker_submenu (window);
+  fill_in_new_term_submenus (window);
+}
